#include <stdio.h>
#include <gtk/gtk.h>
#include "sphereview.h"

static guchar* LoadSphereDB( FILE *in );
static guchar swap_char( guint8 c );
static gint sphere_to_xbm( guint8 *dst, guint8 *src, gint size );

static guint8  *data = NULL;

guint8* GetSphereData( gint slot )
{
  if( data == NULL ) return NULL;
  
  if( 0<=slot && slot<5 )
    return data+3200*slot;
  else 
    return NULL;
}

guint8* LoadSphereDB( FILE *in )
{
  gchar   head[] = "SphereDB";
  gint    c;
  gshort  i,j,size;
  guint8  *pl,*pb,*dp;
  guint8  tmpl[6400];  /* 2*160*20 */
  
  /* First Header */
  for( i=0;i<8;i++ ) {
    c = fgetc( in );
    if( c<0 || (char)c != head[i] ) {
      fprintf( stderr, "It seem Ellegal Database...\nexit.\n" );
      return NULL;
    }
  }
  fseek( in, 80-8, SEEK_CUR );
  
  /* Second Header */
  fseek( in, 40, SEEK_CUR );  /* Skip */
  
  if( data != NULL ) g_free( data );
  data = (guint8*)g_malloc( 3200*5 ); /* 160*20 * 5 */
  dp = data;
  
  for( i=0; i<5; i++ ) {
    size = fread( tmpl, 1, 6400, in );
    if( size!=6400 ) {
      fprintf( stderr, "Can't Read FullData [%d]?!\n", i );
      return NULL;
    }
    pl = tmpl;  pb = tmpl+3200;
    dp = data+3200*i;
    for( j=0;j<3200;j++ ) {
      *dp = *pl | *pb;
      pl++; pb++; dp++;
    }
  }
  
  return data;
}

GdkPixmap** create_pix_from_db( GtkWidget *parent, gchar *filename )
{
  GdkPixmap**  pixs;
  guint8       *pixdata,xbmdata[3200];
  
  GtkStyle   *style;
  gshort     i;
  
  FILE  *file;
  
  file = fopen( filename, "rb" );
  if( file == NULL ) {
    fprintf( stderr, "Can't Load PDB File!!\n" );
    return NULL;
  }
  pixdata = LoadSphereDB( file );
  fclose( file );
  
  if( pixdata == NULL ) { return NULL; }
  
  style = gtk_widget_get_style( parent );
  
  pixs = (GdkPixmap**)g_malloc( sizeof(GdkPixmap*)*5 );
  for( i=0;i<5;i++ ) {
    sphere_to_xbm( xbmdata, pixdata+3200*i, 3200 );
    pixs[i] = gdk_pixmap_create_from_data( parent->window,
					   xbmdata,
					   160, 160, -1,
					   &style->black,
					   &style->white );
  }
  
  return pixs;
}

guint8 swap_char( guint8 c )
{
  guint8  r;
  
  r = 0;
  if( c & 0x80 ) r|=0x01;
  if( c & 0x40 ) r|=0x02;
  if( c & 0x20 ) r|=0x04;
  if( c & 0x10 ) r|=0x08;
  if( c & 0x08 ) r|=0x10;
  if( c & 0x04 ) r|=0x20;
  if( c & 0x02 ) r|=0x40;
  if( c & 0x01 ) r|=0x80;
  
  return r;
}

gint sphere_to_xbm( guint8 *dst, guint8 *src, gint size )
{
  gint    i;
  
  for( i=0;i<size;i++ ) {
    *dst = swap_char( *src );
    dst++; src++;
  }
  
  return 0;
}
