#include <stdio.h>
#include <gtk/gtk.h>
#include "sphereview.h"


#include  "spv_exit.xpm"
#include  "spv_left.xpm"
#include  "spv_right.xpm"
#include  "spv_open.xpm"
#include  "spv_save.xpm"
#define SPV_EXIT   spv_exit_xpm
#define SPV_LEFT   spv_left_xpm
#define SPV_RIGHT  spv_right_xpm
#define SPV_OPEN   spv_open_xpm
#define SPV_SAVE   spv_save_xpm

static GtkWidget* xpm_button( GtkWidget *parent, gchar *filename );
static GtkWidget* xpm_button_d( GtkWidget *parent, gchar **xpm_data );

static void spv_exit( GtkWidget *widget, gpointer *data );
static void spv_move( GtkWidget *widget, gpointer *data );
static void spv_opendb( GtkWidget *widget, GtkWidget *data );
static void spv_savebmp( GtkWidget *widget, gpointer *data );

static void opendb_ok_sel( GtkWidget *widget, GtkWidget *filew );
static void savebmp_ok_sel( GtkWidget *widget, GtkWidget *filew );

static GtkWidget *save_bmp_button;

GtkWidget* xpm_button( GtkWidget *parent, gchar *filename )
{
  GtkWidget *button;
  GtkWidget *pixmapwidget;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  GtkStyle  *style;
  
  button = gtk_button_new();
  
  style = gtk_widget_get_style( parent );
  
  pixmap = gdk_pixmap_create_from_xpm( parent->window,
				       &mask,
				       &style->bg[GTK_STATE_NORMAL],
				       filename );
  
  pixmapwidget = gtk_pixmap_new( pixmap, mask );
  
  gtk_container_add( GTK_CONTAINER(button), pixmapwidget );
  
  gtk_widget_show( pixmapwidget );
  
  return button;
}

GtkWidget* xpm_button_d( GtkWidget *parent, gchar **xpm_data )
{
  GtkWidget *button;
  GtkWidget *pixmapwidget;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  GtkStyle  *style;
  
  button = gtk_button_new();
  
  style = gtk_widget_get_style( parent );
  
  pixmap = gdk_pixmap_create_from_xpm_d( parent->window,
					 &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 xpm_data );
  
  pixmapwidget = gtk_pixmap_new( pixmap, mask );
  
  gtk_container_add( GTK_CONTAINER(button), pixmapwidget );
  
  gtk_widget_show( pixmapwidget );
  
  return button;
}

GtkWidget *exit_button( GtkWidget *parent )
{
  GtkWidget *btn;
  
  btn = xpm_button_d( parent, SPV_EXIT );
  gtk_signal_connect( GTK_OBJECT(btn), "clicked",
		      GTK_SIGNAL_FUNC(spv_exit), NULL );
  
  return btn;
}

GtkWidget *allow_button( GtkWidget *parent, gint dir )
{
  GtkWidget *btn;
  
  btn = xpm_button_d( parent,
		    (dir==1)?SPV_LEFT:SPV_RIGHT );
  gtk_signal_connect( GTK_OBJECT(btn), "clicked",
		      GTK_SIGNAL_FUNC(spv_move),
		      (gpointer)( (dir==1)?"left":"right" ) );
  
  return btn;
}

GtkWidget *opendb_button( GtkWidget *parent )
{
  GtkWidget *btn;
  
  btn = xpm_button_d( parent, SPV_OPEN );
  gtk_signal_connect( GTK_OBJECT(btn), "clicked",
		      GTK_SIGNAL_FUNC(spv_opendb), parent );
  
  return btn;
}

GtkWidget *savebmp_button( GtkWidget *parent )
{
  GtkWidget *btn;
  
  btn = xpm_button_d( parent, SPV_SAVE );
  gtk_signal_connect( GTK_OBJECT(btn), "clicked",
		      GTK_SIGNAL_FUNC(spv_savebmp), parent );
  gtk_widget_set_sensitive( btn, FALSE );
  save_bmp_button = btn;
  
  return btn;
}

/* 終了 */
void spv_exit( GtkWidget *widget, gpointer *data )
{
  //printf( "Exit Viewer\n" );
  gtk_main_quit();
}

void spv_move( GtkWidget *widget, gpointer *data )
{
  if( *((gchar*)data) == 'l' ) {
    spv_page_translate( -1 );
  }
  else {
    spv_page_translate( 1 );
  }
}

void spv_opendb( GtkWidget *widget, GtkWidget *parent )
{
  GtkWidget *filew;
  
  filew = gtk_file_selection_new ("Open SphereDB");
  
  gtk_signal_connect (GTK_OBJECT (filew), "destroy",
                        GTK_SIGNAL_FUNC (gtk_main_quit), NULL);

  /* ok_button の callback 指定 */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		      "clicked", GTK_SIGNAL_FUNC(opendb_ok_sel), filew );
  
  /* cancel_button の callback 指定 */
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button),
  			     "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
  			     GTK_OBJECT(filew));
  
  gtk_window_set_modal( GTK_WINDOW(filew), TRUE);
  gtk_window_set_transient_for( GTK_WINDOW(filew), GTK_WINDOW(parent) );

  gtk_widget_show( filew );
  
  gtk_main();
}

void opendb_ok_sel( GtkWidget *widget, GtkWidget *filew )
{
  gchar  *filename;

  filename = gtk_file_selection_get_filename( GTK_FILE_SELECTION(filew) );
  //g_print( "Load DB %s\n", filename );
  
  spv_pixview_load_db( filename );
  gtk_widget_destroy( filew );
  
  gtk_widget_set_sensitive( save_bmp_button, TRUE );
}

void spv_savebmp( GtkWidget *widget, gpointer *parent )
{
  GtkWidget  *filew;
  
  filew = gtk_file_selection_new( "Save BMP" );
  
  gtk_signal_connect (GTK_OBJECT (filew), "destroy",
                        GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
  
   /* ok_button の callback 指定 */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		      "clicked", GTK_SIGNAL_FUNC(savebmp_ok_sel), filew );
  
  /* cancel_button の callback 指定 */
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button),
  			     "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
  			     GTK_OBJECT(filew));
  
  gtk_window_set_modal( GTK_WINDOW(filew), TRUE);
  gtk_window_set_transient_for( GTK_WINDOW(filew), GTK_WINDOW(parent) );

  gtk_widget_show( filew );
  
  gtk_main();
}

void savebmp_ok_sel( GtkWidget *widget, GtkWidget *filew )
{
  gchar  *filename;
  guint8 *data;
  
  filename = gtk_file_selection_get_filename( GTK_FILE_SELECTION(filew) );
  //g_print( "Save BMP File %s\n", filename );
  
  data = GetSphereData( spv_get_page() );
  WriteBMPFile( filename, data );
  
  gtk_widget_destroy( filew );
}
